@@ -248,6 +248,57 @@ def dutycycle3d(df,speed_name, torque_name, barnum, title):
     plt.show()
 
 
+# bsfcmap + dutycycle
+def bsfcmap_dutycycle(df, speed_name, torque_name, bsfc_name, stepx, stepy, bubblesize, title, level):
+    from scipy.interpolate import griddata
+    import matplotlib.pyplot as plt
+    import pandas as pd
+    import numpy as np
+
+    df = df
+    x = np.array(df[speed_name])
+    y = np.array(df[torque_name])
+    z = np.array(df[bsfc_name])
+    level = level
+    title = title
+
+    xi = np.linspace(min(x), max(x), 1000)
+    yi = np.linspace(min(y), max(y), 1000)
+    X, Y = np.meshgrid(xi, yi)
+    Z = griddata((x, y), z, (X, Y), method='cubic')
+
+    # duty cycle inputs:
+    x1 = int((min(x) - 200) / 100) * 100
+    x2 = int((max(x) + 200) / 100) * 100
+    y1 = int((min(y) - 200) / 100) * 100
+    y2 = int((max(y) + 200) / 100) * 100
+
+    xedges = [*range(x1, x2, stepx)]
+    yedges = [*range(y1, y2, stepy)]
+
+    hist_temp, xedges, yedges = np.histogram2d(x, y, bins=(xedges, yedges))
+    hist = hist_temp.T
+    xpos, ypos = np.meshgrid(xedges[:-1], yedges[:-1])
+    xpos = xpos.ravel()
+    ypos = ypos.ravel()
+
+    # Visualization(duty cycle+fuel map) method: nearest / linear / cubic
+    # (fuel map)
+    plt.figure(figsize=(12, 8))
+    C = plt.contour(X, Y, Z, level, colors='black', alpha=0.5)
+    plt.contourf(X, Y, Z, level, alpha=.75, cmap='plasma')
+    plt.colorbar()
+    plt.clabel(C, inline=True, fontsize=10, colors='black')
+    plt.xlabel(speed_name)
+    plt.ylabel(torque_name)
+    plt.title(title)
+
+    # (duty cycle)
+    plt.grid(which='both', color='grey', linestyle='--', alpha=0.6)
+    plt.scatter(xpos, ypos, s=hist * bubblesize, alpha=0.3, color='white', linewidth=1, edgecolor='black')
+    plt.show()
+
+
 # boxplot01 (single boxplot, 1 parameter, details info) | 20.dec.14
 def boxplot01(df, parameter_name, title):
     import matplotlib.pyplot as plt
@@ -424,12 +475,10 @@ def dfprofile(df):
     import webbrowser
 
     df = df
-    profile = ProfileReport(df, title='SSAP - Dataset Profile Report', html={'style': {'full_width': True}})
-
+    profile = ProfileReport(df, title='{©Tony-lab}Data Profile Report', html={'style': {'full_width': True}})
     profile.to_file("profile_file.html")
     url = 'profile_file.html'
     webbrowser.open(url, new=2)
-
     return profile
 
 
@@ -917,13 +966,30 @@ def pie(df, parameter, title):
     plt.show()
 
 
+# heatmap - show correlation of each parameter
+def heatmap(df, title):
+    import seaborn as sns
+    import numpy as np
+    import matplotlib.pyplot as plt
+
+    df = df
+
+    plt.figure(figsize=(12, 8))
+    sns.heatmap(df.corr(), xticklabels=df.corr().columns, yticklabels=df.corr().columns, linewidth=1, linecolor='white',
+                cmap='Blues', center=0, annot=True)
+    plt.title(title)
+    plt.xticks(rotation=30)
+    plt.yticks(rotation=30)
+    plt.show()
+
+
 # scatter chart 01 - basics
 def scatter01(df, parameter_x, parameter_y, title):
     df = df
     x = df[parameter_x].tolist()
     y = df[parameter_y].tolist()
     fig, ax = plt.subplots()
-    scatter = ax.scatter(x=x, y=y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='orange')
+    scatter = ax.scatter(x=x, y=y, s=500, edgecolor='black', lw=0.5, alpha=0.7, facecolor='orange')
     plt.grid(linestyle='--', lw=0.5)
 
     plt.xlabel(parameter_x)
@@ -978,7 +1044,7 @@ def scatter03(df, parameter_x, parameter_y, category, title):
     for i in catlist:
         x = df[parameter_x].loc[df[category] == i].tolist()
         y = df[parameter_y].loc[df[category] == i].tolist()
-        scatter = ax.scatter(x=x, y=y, c=sliceColors[k], label=i, edgecolor='black', lw=0.5, alpha=0.7,
+        scatter = ax.scatter(x=x, y=y, s=400, c=sliceColors[k], label=i, edgecolor='black', lw=0.5, alpha=0.7,
                              facecolor='orange')
         k += 1
 
@@ -994,14 +1060,402 @@ def scatter03(df, parameter_x, parameter_y, category, title):
     plt.show()
 
 
+# scatter plot + hist plot on top/right.
+def scatter04hist(df, parameter_x, parameter_y, bins, title):
+    import matplotlib.pyplot as plt
+
+    df = df
+    x = df[parameter_x]
+    y = df[parameter_y]
+    title = title
+
+    # definitions for the axes
+    left, width = 0.1, 0.65
+    bottom, height = 0.1, 0.65
+    spacing = 0.005
+
+    rect_scatter = [left, bottom, width, height]
+    rect_histx = [left, bottom + height + spacing, width, 0.2]
+    rect_histy = [left + width + spacing, bottom, 0.2, height]
+
+    # start with a rectangular Figure
+    plt.figure(figsize=(8, 8))
+
+    # scatter plot
+    ax_scatter = plt.axes(rect_scatter)
+    plt.grid(linestyle='--', lw=0.5)
+    ax_scatter.tick_params(direction='in', top=True, right=True)
+    plt.xlabel(parameter_x)
+    plt.ylabel(parameter_y)
+
+    # top hist
+    ax_histx = plt.axes(rect_histx)
+    plt.grid(linestyle='--', lw=0.5, axis='y')
+    ax_histx.tick_params(direction='in', labelbottom=False)
+    plt.xlabel(parameter_x)
+    plt.ylabel('Counts')
+    plt.title(title)
+
+    # right hist
+    ax_histy = plt.axes(rect_histy)
+    plt.grid(linestyle='--', lw=0.5, axis='x')
+    ax_histy.tick_params(direction='in', labelleft=False)
+    plt.xlabel('Counts')
+    plt.ylabel(parameter_y)
+
+    # the scatter plot:
+    ax_scatter.scatter(x, y, s=400, edgecolor='black', lw=0.5, alpha=0.7, facecolor='darkorange')
+
+    bins = bins
+
+    # now determine nice limits by hand:
+    ax_histx.hist(x, bins=bins, edgecolor='black', facecolor='darkorange', alpha=0.7)
+    ax_histy.hist(y, bins=bins, orientation='horizontal', edgecolor='black', facecolor='darkorange', alpha=0.7)
+
+    ax_histx.set_xlim(ax_scatter.get_xlim())
+    #     ax_histy.set_ylim(ax_scatter.get_ylim())
+
+    plt.show()
+
+
+# scatter plot + box plot on top/right.
+def scatter05box(df, parameter_x, parameter_y, title):
+    import matplotlib.pyplot as plt
+
+    df = df
+    x = df[parameter_x]
+    y = df[parameter_y]
+    title = title
+
+    # definitions for the axes
+    left, width = 0.1, 0.65
+    bottom, height = 0.1, 0.65
+    spacing = 0.005
+
+    rect_scatter = [left, bottom, width, height]
+    rect_histx = [left, bottom + height + spacing, width, 0.08]
+    rect_histy = [left + width + spacing, bottom, 0.08, height]
+
+    # start with a rectangular Figure
+    plt.figure(figsize=(8, 8))
+
+    # scatter plot
+    ax_scatter = plt.axes(rect_scatter)
+    plt.grid(linestyle='--', lw=0.5)
+    ax_scatter.tick_params(direction='in', top=True, right=True)
+    plt.xlabel(parameter_x)
+    plt.ylabel(parameter_y)
+
+    # top hist
+    ax_histx = plt.axes(rect_histx)
+    plt.grid(linestyle='--', lw=0.5, axis='y')
+    ax_histx.tick_params(direction='in', labelbottom=False)
+    plt.xlabel(parameter_x)
+    #     plt.ylabel('Counts')
+    plt.title(title)
+
+    # right hist
+    ax_histy = plt.axes(rect_histy)
+    plt.grid(linestyle='--', lw=0.5, axis='x')
+    ax_histy.tick_params(direction='in', labelleft=False)
+    #     plt.xlabel('Counts')
+    plt.ylabel(parameter_y)
+
+    # the scatter plot:
+    ax_scatter.scatter(x, y, s=500, edgecolor='black', lw=0.5, alpha=0.7, facecolor='darkorange')
+
+    # now determine nice limits by hand:
+    #     ax_histx.hist(x, bins=bins,edgecolor='black',facecolor='darkorange',alpha=0.7)
+    #     ax_histy.hist(y, bins=bins, orientation='horizontal',edgecolor='black',facecolor='darkorange',alpha=0.7)
+
+    red_square = dict(markerfacecolor='lightgrey', marker='o', alpha=0.5, lw=1)
+
+    x = ax_histx.boxplot(x, flierprops=red_square, vert=False, widths=0.5, patch_artist=True)
+    for patch in x['boxes']:
+        patch.set_facecolor('orange')
+
+    y = ax_histy.boxplot(y, flierprops=red_square, widths=0.5, patch_artist=True)
+    for patch in y['boxes']:
+        patch.set_facecolor('orange')
+
+    #     ax_histx.set_xlim(ax_scatter.get_xlim())
+    #     ax_histy.set_ylim(ax_scatter.get_ylim())
+
+    plt.show()
+
+
+# scatter plot + hist plot + linear regression
+def scatter06histlr(df, parameter_x, parameter_y, bins, title):
+    import matplotlib.pyplot as plt
+    from scipy.stats import linregress
+    import numpy as np
+
+    df = df
+    x = df[parameter_x]
+    y = df[parameter_y]
+    lr = linregress(x, y)
+    title = title
+
+    lr_label = '[L.R] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
+        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
+
+    # definitions for the axes
+    left, width = 0.1, 0.65
+    bottom, height = 0.1, 0.65
+    spacing = 0.005
+
+    rect_scatter = [left, bottom, width, height]
+    rect_histx = [left, bottom + height + spacing, width, 0.2]
+    rect_histy = [left + width + spacing, bottom, 0.2, height]
+
+    # start with a rectangular Figure
+    plt.figure(figsize=(8, 8))
+
+    # scatter plot
+    ax_scatter = plt.axes(rect_scatter)
+    plt.grid(linestyle='--', lw=0.5)
+    ax_scatter.tick_params(direction='in', top=True, right=True)
+    plt.xlabel(parameter_x)
+    plt.ylabel(parameter_y)
+
+    # top hist
+    ax_histx = plt.axes(rect_histx)
+    plt.grid(linestyle='--', lw=0.5, axis='y')
+    ax_histx.tick_params(direction='in', labelbottom=False)
+    plt.xlabel(parameter_x)
+    plt.ylabel('Counts')
+    plt.title(title)
+
+    # right hist
+    ax_histy = plt.axes(rect_histy)
+    plt.grid(linestyle='--', lw=0.5, axis='x')
+    ax_histy.tick_params(direction='in', labelleft=False)
+    plt.xlabel('Counts')
+    plt.ylabel(parameter_y)
+
+    # the scatter plot:
+    ax_scatter.scatter(x, y, s=300, edgecolor='black', lw=0.5, alpha=0.7, facecolor='darkorange')
+
+    coef = np.polyfit(x, y, 1)
+    poly1d_fn = np.poly1d(coef)
+    ax_scatter.plot(x, poly1d_fn(x), '-', lw=1, color='blue', label=lr_label)
+    ax_scatter.legend(loc='lower left')
+
+    bins = bins
+
+    # now determine nice limits by hand:
+    ax_histx.hist(x, bins=bins, edgecolor='black', facecolor='darkorange', alpha=0.7)
+    ax_histy.hist(y, bins=bins, orientation='horizontal', edgecolor='black', facecolor='darkorange', alpha=0.7)
+
+    ax_histx.set_xlim(ax_scatter.get_xlim())
+    #     ax_histy.set_ylim(ax_scatter.get_ylim())
+
+    plt.show()
+
+
+# scatter plot + boxplot + linear regression
+def scatter07boxlr(df, parameter_x, parameter_y, title):
+    import matplotlib.pyplot as plt
+    import numpy as np
+    from scipy.stats import linregress
+
+    df = df
+    x = df[parameter_x]
+    y = df[parameter_y]
+
+    lr = linregress(x, y)
+    title = title
+
+    lr_label = '[L.R] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
+        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
+    #     print(lr_label)
+
+    # definitions for the axes
+    left, width = 0.1, 0.65
+    bottom, height = 0.1, 0.65
+    spacing = 0.005
+
+    rect_scatter = [left, bottom, width, height]
+    rect_histx = [left, bottom + height + spacing, width, 0.08]
+    rect_histy = [left + width + spacing, bottom, 0.08, height]
+
+    # start with a rectangular Figure
+    plt.figure(figsize=(8, 8))
+
+    # scatter plot
+    ax_scatter = plt.axes(rect_scatter)
+    plt.grid(linestyle='--', lw=0.5)
+    ax_scatter.tick_params(direction='in', top=True, right=True)
+    plt.xlabel(parameter_x)
+    plt.ylabel(parameter_y)
+
+    # top hist
+    ax_histx = plt.axes(rect_histx)
+    plt.grid(linestyle='--', lw=0.5, axis='y')
+    ax_histx.tick_params(direction='in', labelbottom=False)
+    plt.xlabel(parameter_x)
+    #     plt.ylabel('Counts')
+    plt.title(title)
+
+    # right hist
+    ax_histy = plt.axes(rect_histy)
+    plt.grid(linestyle='--', lw=0.5, axis='x')
+    ax_histy.tick_params(direction='in', labelleft=False)
+    #     plt.xlabel('Counts')
+    plt.ylabel(parameter_y)
+    # the scatter plot:
+    ax_scatter.scatter(x, y, s=200, edgecolor='black', lw=0.5, alpha=0.7, facecolor='darkorange')
+
+    #     print(ploy1d_fn(x))
+    coef = np.polyfit(x, y, 1)
+    poly1d_fn = np.poly1d(coef)
+    ax_scatter.plot(x, poly1d_fn(x), '-', lw=1, color='blue', label=lr_label)
+    ax_scatter.legend(loc='lower left')
+
+    # now determine nice limits by hand:
+    #     ax_histx.hist(x, bins=bins,edgecolor='black',facecolor='darkorange',alpha=0.7)
+    #     ax_histy.hist(y, bins=bins, orientation='horizontal',edgecolor='black',facecolor='darkorange',alpha=0.7)
+
+    red_square = dict(markerfacecolor='lightgrey', marker='o', alpha=0.5, lw=1)
+
+    x = ax_histx.boxplot(x, flierprops=red_square, vert=False, widths=0.5, patch_artist=True)
+    for patch in x['boxes']:
+        patch.set_facecolor('orange')
+
+    y = ax_histy.boxplot(y, flierprops=red_square, widths=0.5, patch_artist=True)
+    for patch in y['boxes']:
+        patch.set_facecolor('orange')
+
+    #     ax_histx.set_xlim(ax_scatter.get_xlim())
+    #     ax_histy.set_ylim(ax_scatter.get_ylim())
+
+    plt.show()
+
+
+# scatter plot + linear regression + 95%ci error zone.
+def scatter08lrci(df, parameter_x, parameter_y, title):
+    import numpy as np
+    import matplotlib.pyplot as plt
+    from scipy import stats
+    from scipy.stats import linregress
+
+    df = df
+    x = df[parameter_x].tolist()
+    y = df[parameter_y].tolist()
+    coef = np.polyfit(x, y, 1)
+    poly1d_fn = np.poly1d(coef)
+
+    lr = linregress(x, y)
+    lr_label = '[lr] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
+        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
+
+    sample = y - poly1d_fn(x)
+
+    #     print(x)
+
+    def mean_ci(level, sample):
+        confidence_level = level
+        degrees_freedom = sample.size - 1
+        # df:degree of freedom, df = N-k (N-sample number, k-restricted condition/variables).
+        # df=N-1 (restricted condition, mean=80). more df → more evidence → more convincible conslusion.
+        sample_mean = np.mean(sample)
+        sample_mean_standard_error = stats.sem(sample)
+        confidence_interval = stats.t.interval(confidence_level, degrees_freedom, sample_mean,
+                                               sample_mean_standard_error)
+        # got small number sample case, use t-test.
+        return sample_mean, confidence_interval[0], confidence_interval[1]
+
+    sample_mean, lower, upper = mean_ci(level=0.95, sample=sample)
+
+    x1 = list(set(x))
+    x1.sort()
+
+    est_upper = poly1d_fn(x1) + sample_mean + upper
+    est_lower = poly1d_fn(x1) + sample_mean + lower
+    #     print(x1)
+    #     print(est_upper)
+
+    fig, ax = plt.subplots()
+
+    scatter = ax.scatter(x=x, y=y, s=100, edgecolor='black', lw=0.5, alpha=0.7, facecolor='orange')
+    plt.plot(x, poly1d_fn(x), '-', color='blue', lw=0.5, label=lr_label)
+    #     plt.scatter(x, est_upper, c = 'b')
+    #     plt.scatter(x, est_lower, c = 'b')
+    plt.fill_between(x1, est_upper, est_lower, color='lightblue', alpha=0.5, label='[ci] 95% zone')
+
+    plt.grid(linestyle='-', lw=0.5)
+
+    plt.xlabel(parameter_x)
+    plt.ylabel(parameter_y)
+    plt.legend(loc='upper left')
+
+    plt.title(title)
+    plt.show()
+
+
+# scatter plot + linear regression + ±n*sigma zone.
+def scatter09lr6sigma(df, parameter_x, parameter_y, sigma, title):
+    import numpy as np
+    import matplotlib.pyplot as plt
+    from scipy import stats
+    from scipy.stats import linregress
+
+    df = df
+    x = df[parameter_x].tolist()
+    y = df[parameter_y].tolist()
+    coef = np.polyfit(x, y, 1)
+    poly1d_fn = np.poly1d(coef)
+
+    lr = linregress(x, y)
+    lr_label = '[lr] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
+        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
+
+    sample = y - poly1d_fn(x)
+    sample_mean = np.mean(sample)
+    sample_stderr = np.std(sample)
+
+    x1 = list(set(x))
+    x1.sort()
+    n = sigma
+
+    est_upper = poly1d_fn(x1) + sample_mean + n * sample_stderr
+    est_lower = poly1d_fn(x1) + sample_mean - n * sample_stderr
+
+    fig, ax = plt.subplots()
+
+    scatter = ax.scatter(x=x, y=y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='orange')
+    plt.plot(x, poly1d_fn(x), '-', color='blue', lw=0.5, label=lr_label)
+
+    if n == 1:
+        cover = '65.26%'
+    elif n == 2:
+        cover = '95.44%'
+    elif n == 3:
+        cover = '99.74%'
+    else:
+        cover = '?, sigma should be 1,2 or 3.'
+
+    plt.fill_between(x1, est_upper, est_lower, color='lightblue', alpha=0.5,
+                     label='±' + str(n) + ' sigma zone (cover ' + cover + ')')
+
+    plt.grid(linestyle='-', lw=0.5)
+
+    plt.xlabel(parameter_x)
+    plt.ylabel(parameter_y)
+    plt.legend(loc='upper left')
+
+    plt.title(title)
+    plt.show()
+
+
 # empty map:
 def map():
     import folium
     import webbrowser
 
     m = folium.Map(location=[39.9, 116.4])
-    m.save('ssap_map.html')
-    url = 'ssap_map.html'
+    m.save('tonylab_map.html')
+    url = 'tonylab_map.html'
     webbrowser.open(url, new=2)
     return m
 
@@ -1053,8 +1507,8 @@ def map01(df, province, parameter_x, method):
 
     print(dfnew)
 
-    pdsmap = folium.Map(location=[35, 120], zoom_start=4)
-    pdsmap.choropleth(
+    labmap = folium.Map(location=[35, 120], zoom_start=4)
+    labmap.choropleth(
         geo_data=open('china.json', encoding='utf-8-sig').read(),
         data=dfnew,
         columns=['province', 'value'],
@@ -1065,74 +1519,16 @@ def map01(df, province, parameter_x, method):
         nan_fill_opacity=0.1,
         line_color='grey',
         line_opacity=0.2,
-        legend_name='[PDS_Map] distribution for: ' + parameter_x + ' (' + method + ')',
+        legend_name='[©TonyLab] Geo_loc_distribution of: ' + parameter_x + ' (' + method + ')',
         highlight=True,
     )
 
-    pdsmap.save('PDS_Map.html')
-    url = 'PDS_Map.html'
+    labmap.save('Tonylab_Map.html')
+    url = 'Tonylab_Map.html'
     webbrowser.open(url, new=2)
     # display(pdsmap)
 
 
-# scatter plot + hist plot on top/right.
-def scatter04hist(df, parameter_x, parameter_y, bins, title):
-    import matplotlib.pyplot as plt
-
-    df = df
-    x = df[parameter_x]
-    y = df[parameter_y]
-    title = title
-
-    # definitions for the axes
-    left, width = 0.1, 0.65
-    bottom, height = 0.1, 0.65
-    spacing = 0.005
-
-    rect_scatter = [left, bottom, width, height]
-    rect_histx = [left, bottom + height + spacing, width, 0.2]
-    rect_histy = [left + width + spacing, bottom, 0.2, height]
-
-    # start with a rectangular Figure
-    plt.figure(figsize=(8, 8))
-
-    # scatter plot
-    ax_scatter = plt.axes(rect_scatter)
-    plt.grid(linestyle='--', lw=0.5)
-    ax_scatter.tick_params(direction='in', top=True, right=True)
-    plt.xlabel(parameter_x)
-    plt.ylabel(parameter_y)
-
-    # top hist
-    ax_histx = plt.axes(rect_histx)
-    plt.grid(linestyle='--', lw=0.5, axis='y')
-    ax_histx.tick_params(direction='in', labelbottom=False)
-    plt.xlabel(parameter_x)
-    plt.ylabel('Counts')
-    plt.title(title)
-
-    # right hist
-    ax_histy = plt.axes(rect_histy)
-    plt.grid(linestyle='--', lw=0.5, axis='x')
-    ax_histy.tick_params(direction='in', labelleft=False)
-    plt.xlabel('Counts')
-    plt.ylabel(parameter_y)
-
-    # the scatter plot:
-    ax_scatter.scatter(x, y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='darkorange')
-
-    bins = bins
-
-    # now determine nice limits by hand:
-    ax_histx.hist(x, bins=bins, edgecolor='black', facecolor='darkorange', alpha=0.7)
-    ax_histy.hist(y, bins=bins, orientation='horizontal', edgecolor='black', facecolor='darkorange', alpha=0.7)
-
-    ax_histx.set_xlim(ax_scatter.get_xlim())
-    #     ax_histy.set_ylim(ax_scatter.get_ylim())
-
-    plt.show()
-
-
 # show colormap detail:
 def colormap():
     import numpy as np
@@ -1266,336 +1662,6 @@ def colorlist():
     plt.show()
 
 
-# scatter plot + box plot on top/right.
-def scatter05box(df, parameter_x, parameter_y, title):
-    import matplotlib.pyplot as plt
-
-    df = df
-    x = df[parameter_x]
-    y = df[parameter_y]
-    title = title
-
-    # definitions for the axes
-    left, width = 0.1, 0.65
-    bottom, height = 0.1, 0.65
-    spacing = 0.005
-
-    rect_scatter = [left, bottom, width, height]
-    rect_histx = [left, bottom + height + spacing, width, 0.08]
-    rect_histy = [left + width + spacing, bottom, 0.08, height]
-
-    # start with a rectangular Figure
-    plt.figure(figsize=(8, 8))
-
-    # scatter plot
-    ax_scatter = plt.axes(rect_scatter)
-    plt.grid(linestyle='--', lw=0.5)
-    ax_scatter.tick_params(direction='in', top=True, right=True)
-    plt.xlabel(parameter_x)
-    plt.ylabel(parameter_y)
-
-    # top hist
-    ax_histx = plt.axes(rect_histx)
-    plt.grid(linestyle='--', lw=0.5, axis='y')
-    ax_histx.tick_params(direction='in', labelbottom=False)
-    plt.xlabel(parameter_x)
-    #     plt.ylabel('Counts')
-    plt.title(title)
-
-    # right hist
-    ax_histy = plt.axes(rect_histy)
-    plt.grid(linestyle='--', lw=0.5, axis='x')
-    ax_histy.tick_params(direction='in', labelleft=False)
-    #     plt.xlabel('Counts')
-    plt.ylabel(parameter_y)
-
-    # the scatter plot:
-    ax_scatter.scatter(x, y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='darkorange')
-
-    # now determine nice limits by hand:
-    #     ax_histx.hist(x, bins=bins,edgecolor='black',facecolor='darkorange',alpha=0.7)
-    #     ax_histy.hist(y, bins=bins, orientation='horizontal',edgecolor='black',facecolor='darkorange',alpha=0.7)
-
-    red_square = dict(markerfacecolor='lightgrey', marker='o', alpha=0.5, lw=1)
-
-    x = ax_histx.boxplot(x, flierprops=red_square, vert=False, widths=0.5, patch_artist=True)
-    for patch in x['boxes']:
-        patch.set_facecolor('orange')
-
-    y = ax_histy.boxplot(y, flierprops=red_square, widths=0.5, patch_artist=True)
-    for patch in y['boxes']:
-        patch.set_facecolor('orange')
-
-    #     ax_histx.set_xlim(ax_scatter.get_xlim())
-    #     ax_histy.set_ylim(ax_scatter.get_ylim())
-
-    plt.show()
-
-
-# scatter plot + hist plot + linear regression
-def scatter06histlr(df, parameter_x, parameter_y, bins, title):
-    import matplotlib.pyplot as plt
-    from scipy.stats import linregress
-    import numpy as np
-
-    df = df
-    x = df[parameter_x]
-    y = df[parameter_y]
-    lr = linregress(x, y)
-    title = title
-
-    lr_label = '[L.R] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
-        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
-
-    # definitions for the axes
-    left, width = 0.1, 0.65
-    bottom, height = 0.1, 0.65
-    spacing = 0.005
-
-    rect_scatter = [left, bottom, width, height]
-    rect_histx = [left, bottom + height + spacing, width, 0.2]
-    rect_histy = [left + width + spacing, bottom, 0.2, height]
-
-    # start with a rectangular Figure
-    plt.figure(figsize=(8, 8))
-
-    # scatter plot
-    ax_scatter = plt.axes(rect_scatter)
-    plt.grid(linestyle='--', lw=0.5)
-    ax_scatter.tick_params(direction='in', top=True, right=True)
-    plt.xlabel(parameter_x)
-    plt.ylabel(parameter_y)
-
-    # top hist
-    ax_histx = plt.axes(rect_histx)
-    plt.grid(linestyle='--', lw=0.5, axis='y')
-    ax_histx.tick_params(direction='in', labelbottom=False)
-    plt.xlabel(parameter_x)
-    plt.ylabel('Counts')
-    plt.title(title)
-
-    # right hist
-    ax_histy = plt.axes(rect_histy)
-    plt.grid(linestyle='--', lw=0.5, axis='x')
-    ax_histy.tick_params(direction='in', labelleft=False)
-    plt.xlabel('Counts')
-    plt.ylabel(parameter_y)
-
-    # the scatter plot:
-    ax_scatter.scatter(x, y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='darkorange')
-
-    coef = np.polyfit(x, y, 1)
-    poly1d_fn = np.poly1d(coef)
-    ax_scatter.plot(x, poly1d_fn(x), '-', lw=1, color='blue', label=lr_label)
-    ax_scatter.legend(loc='lower left')
-
-    bins = bins
-
-    # now determine nice limits by hand:
-    ax_histx.hist(x, bins=bins, edgecolor='black', facecolor='darkorange', alpha=0.7)
-    ax_histy.hist(y, bins=bins, orientation='horizontal', edgecolor='black', facecolor='darkorange', alpha=0.7)
-
-    ax_histx.set_xlim(ax_scatter.get_xlim())
-    #     ax_histy.set_ylim(ax_scatter.get_ylim())
-
-    plt.show()
-
-
-# scatter plot + boxplot + linear regression
-def scatter07boxlr(df, parameter_x, parameter_y, title):
-    import matplotlib.pyplot as plt
-    import numpy as np
-    from scipy.stats import linregress
-
-    df = df
-    x = df[parameter_x]
-    y = df[parameter_y]
-
-    lr = linregress(x, y)
-    title = title
-
-    lr_label = '[L.R] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
-        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
-    #     print(lr_label)
-
-    # definitions for the axes
-    left, width = 0.1, 0.65
-    bottom, height = 0.1, 0.65
-    spacing = 0.005
-
-    rect_scatter = [left, bottom, width, height]
-    rect_histx = [left, bottom + height + spacing, width, 0.08]
-    rect_histy = [left + width + spacing, bottom, 0.08, height]
-
-    # start with a rectangular Figure
-    plt.figure(figsize=(8, 8))
-
-    # scatter plot
-    ax_scatter = plt.axes(rect_scatter)
-    plt.grid(linestyle='--', lw=0.5)
-    ax_scatter.tick_params(direction='in', top=True, right=True)
-    plt.xlabel(parameter_x)
-    plt.ylabel(parameter_y)
-
-    # top hist
-    ax_histx = plt.axes(rect_histx)
-    plt.grid(linestyle='--', lw=0.5, axis='y')
-    ax_histx.tick_params(direction='in', labelbottom=False)
-    plt.xlabel(parameter_x)
-    #     plt.ylabel('Counts')
-    plt.title(title)
-
-    # right hist
-    ax_histy = plt.axes(rect_histy)
-    plt.grid(linestyle='--', lw=0.5, axis='x')
-    ax_histy.tick_params(direction='in', labelleft=False)
-    #     plt.xlabel('Counts')
-    plt.ylabel(parameter_y)
-    # the scatter plot:
-    ax_scatter.scatter(x, y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='darkorange')
-
-    #     print(ploy1d_fn(x))
-    coef = np.polyfit(x, y, 1)
-    poly1d_fn = np.poly1d(coef)
-    ax_scatter.plot(x, poly1d_fn(x), '-', lw=1, color='blue', label=lr_label)
-    ax_scatter.legend(loc='lower left')
-
-    # now determine nice limits by hand:
-    #     ax_histx.hist(x, bins=bins,edgecolor='black',facecolor='darkorange',alpha=0.7)
-    #     ax_histy.hist(y, bins=bins, orientation='horizontal',edgecolor='black',facecolor='darkorange',alpha=0.7)
-
-    red_square = dict(markerfacecolor='lightgrey', marker='o', alpha=0.5, lw=1)
-
-    x = ax_histx.boxplot(x, flierprops=red_square, vert=False, widths=0.5, patch_artist=True)
-    for patch in x['boxes']:
-        patch.set_facecolor('orange')
-
-    y = ax_histy.boxplot(y, flierprops=red_square, widths=0.5, patch_artist=True)
-    for patch in y['boxes']:
-        patch.set_facecolor('orange')
-
-    #     ax_histx.set_xlim(ax_scatter.get_xlim())
-    #     ax_histy.set_ylim(ax_scatter.get_ylim())
-
-    plt.show()
-
-
-# scatter plot + linear regression + 95%ci error zone.
-def scatter08lrci(df, parameter_x, parameter_y, title):
-    import numpy as np
-    import matplotlib.pyplot as plt
-    from scipy import stats
-    from scipy.stats import linregress
-
-    df = df
-    x = df[parameter_x].tolist()
-    y = df[parameter_y].tolist()
-    coef = np.polyfit(x, y, 1)
-    poly1d_fn = np.poly1d(coef)
-
-    lr = linregress(x, y)
-    lr_label = '[lr] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
-        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
-
-    sample = y - poly1d_fn(x)
-
-    #     print(x)
-
-    def mean_ci(level, sample):
-        confidence_level = level
-        degrees_freedom = sample.size - 1
-        # df:degree of freedom, df = N-k (N-sample number, k-restricted condition/variables).
-        # df=N-1 (restricted condition, mean=80). more df → more evidence → more convincible conslusion.
-        sample_mean = np.mean(sample)
-        sample_mean_standard_error = stats.sem(sample)
-        confidence_interval = stats.t.interval(confidence_level, degrees_freedom, sample_mean,
-                                               sample_mean_standard_error)
-        # got small number sample case, use t-test.
-        return sample_mean, confidence_interval[0], confidence_interval[1]
-
-    sample_mean, lower, upper = mean_ci(level=0.95, sample=sample)
-
-    x1 = list(set(x))
-    x1.sort()
-
-    est_upper = poly1d_fn(x1) + sample_mean + upper
-    est_lower = poly1d_fn(x1) + sample_mean + lower
-    #     print(x1)
-    #     print(est_upper)
-
-    fig, ax = plt.subplots()
-
-    scatter = ax.scatter(x=x, y=y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='orange')
-    plt.plot(x, poly1d_fn(x), '-', color='blue', lw=0.5, label=lr_label)
-    #     plt.scatter(x, est_upper, c = 'b')
-    #     plt.scatter(x, est_lower, c = 'b')
-    plt.fill_between(x1, est_upper, est_lower, color='lightblue', alpha=0.5, label='[ci] 95% zone')
-
-    plt.grid(linestyle='-', lw=0.5)
-
-    plt.xlabel(parameter_x)
-    plt.ylabel(parameter_y)
-    plt.legend(loc='upper left')
-
-    plt.title(title)
-    plt.show()
-
-
-# scatter plot + linear regression + ±n*sigma zone.
-def scatter09lr6sigma(df, parameter_x, parameter_y, sigma, title):
-    import numpy as np
-    import matplotlib.pyplot as plt
-    from scipy import stats
-    from scipy.stats import linregress
-
-    df = df
-    x = df[parameter_x].tolist()
-    y = df[parameter_y].tolist()
-    coef = np.polyfit(x, y, 1)
-    poly1d_fn = np.poly1d(coef)
-
-    lr = linregress(x, y)
-    lr_label = '[lr] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
-        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
-
-    sample = y - poly1d_fn(x)
-    sample_mean = np.mean(sample)
-    sample_stderr = np.std(sample)
-
-    x1 = list(set(x))
-    x1.sort()
-    n = sigma
-
-    est_upper = poly1d_fn(x1) + sample_mean + n * sample_stderr
-    est_lower = poly1d_fn(x1) + sample_mean - n * sample_stderr
-
-    fig, ax = plt.subplots()
-
-    scatter = ax.scatter(x=x, y=y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='orange')
-    plt.plot(x, poly1d_fn(x), '-', color='blue', lw=0.5, label=lr_label)
-
-    if n == 1:
-        cover = '65.26%'
-    elif n == 2:
-        cover = '95.44%'
-    elif n == 3:
-        cover = '99.74%'
-    else:
-        cover = '?, sigma should be 1,2 or 3.'
-
-    plt.fill_between(x1, est_upper, est_lower, color='lightblue', alpha=0.5,
-                     label='±' + str(n) + ' sigma zone (cover ' + cover + ')')
-
-    plt.grid(linestyle='-', lw=0.5)
-
-    plt.xlabel(parameter_x)
-    plt.ylabel(parameter_y)
-    plt.legend(loc='upper left')
-
-    plt.title(title)
-    plt.show()
-
-
 # normality test. input target list is enough
 def normaltest(targetlist):
     import numpy as np
@@ -1637,23 +1703,6 @@ def parallel(df, category, title):
     plt.show()
 
 
-# heatmap - show correlation of each parameter
-def heatmap(df, title):
-    import seaborn as sns
-    import numpy as np
-    import matplotlib.pyplot as plt
-
-    df = df
-
-    plt.figure(figsize=(12, 8))
-    sns.heatmap(df.corr(), xticklabels=df.corr().columns, yticklabels=df.corr().columns, linewidth=1, linecolor='white',
-                cmap='Blues', center=0, annot=True)
-    plt.title(title)
-    plt.xticks(rotation=30)
-    plt.yticks(rotation=30)
-    plt.show()
-
-
 # matrix plot: 1.scatter + hist; 2.scatter + kde; 3.kde + kde
 def matrix01(df, other_matrix_type, upperleft_diag_matrix_type):
     import seaborn as sns
@@ -1773,7 +1822,7 @@ def line04(df, parameterx, parametery, std_method, gap, title):
     plt.show()
 
 
-# network plot - show relationship between different entities
+# network plot - show relationship between different entities. spiral/circular/other
 def network01(df, entity1, entity2, relation, nodesize, fontsize, layout):
     #   ## show all the relations
 
@@ -1836,7 +1885,6 @@ def network02(df, entity1, entity2, relation):
     #     ]
     # }
 
-
     got_net = Network(height="500px", width="500px", bgcolor="white", font_color="black", notebook=False, directed=True)
     # got_net = Network(notebook=True, directed=True)
 
@@ -1864,69 +1912,104 @@ def network02(df, entity1, entity2, relation):
         node["title"] += " Neighbors:<br>" + "<br>".join(neighbor_map[node["id"]])
     #    node["value"] = len(neighbor_map[node["id"]])
 
-    return got_net.show("KG_Networks.html")
+    return got_net.show("Tonylab_networks.html")
 
 
 
-# bsfcmap + dutycycle
-def bsfcmap_dutycycle(df, speed_name, torque_name, bsfc_name, stepx, stepy, bubblesize, title, level):
-    from scipy.interpolate import griddata
-    import matplotlib.pyplot as plt
-    import pandas as pd
+
+# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
+# Test area (update by 2021/4/26):==============================================
+# df=pd.read_csv('C:/Users/lx379/Python_Tony/tony_lab/showcase_data/network.csv')
+# network02(df, 'entity1', 'entity2', 'relation')
+
+
+def polynominalreg(df,x_parameter,y_parameter,M,title):
+
     import numpy as np
+    import pandas as pd
+    import scipy as sp
+    from scipy.optimize import leastsq
+    import matplotlib.pyplot as plt
 
-    df = df
-    x = np.array(df[speed_name])
-    y = np.array(df[torque_name])
-    z = np.array(df[bsfc_name])
-    level = level
-    title = title
+#     # 目标函数(ideal)
+#     def real_func(x):
+#         return np.sin(2 * np.pi * x)
+    df=df
+    # 多项式
+    def fit_func(p, x):
+        f = np.poly1d(p)
+        return f(x)
 
-    xi = np.linspace(min(x), max(x), 1000)
-    yi = np.linspace(min(y), max(y), 1000)
-    X, Y = np.meshgrid(xi, yi)
-    Z = griddata((x, y), z, (X, Y), method='cubic')
+    # 残差
+    def residuals_func(p, x, y):
+        ret = fit_func(p, x) - y
+        return ret
 
-    # duty cycle inputs:
-    x1 = int((min(x) - 200) / 100) * 100
-    x2 = int((max(x) + 200) / 100) * 100
-    y1 = int((min(y) - 200) / 100) * 100
-    y2 = int((max(y) + 200) / 100) * 100
+    # 确定实际X个点的个数
+#     x = np.linspace(0, 1, 20)
+    x=df[x_parameter]
+    y=df[y_parameter]
+    d=max(x)
+    x_points = np.linspace(0, d, 1000)
 
-    xedges = [*range(x1, x2, stepx)]
-    yedges = [*range(y1, y2, stepy)]
+    # 加上正态分布噪音的目标函数的值
+#     y_ = real_func(x)
+#     y = [np.random.normal(0, 0.2) + y1 for y1 in y_]
 
-    hist_temp, xedges, yedges = np.histogram2d(x, y, bins=(xedges, yedges))
-    hist = hist_temp.T
-    xpos, ypos = np.meshgrid(xedges[:-1], yedges[:-1])
-    xpos = xpos.ravel()
-    ypos = ypos.ravel()
+    # M    为 多项式的次数
+    # """
+    # 随机初始化多项式参数
+    p_init = np.random.rand(M + 1)
 
-    # Visualization(duty cycle+fuel map) method: nearest / linear / cubic
-    # (fuel map)
-    plt.figure(figsize=(12, 8))
-    C = plt.contour(X, Y, Z, level, colors='black', alpha=0.5)
-    plt.contourf(X, Y, Z, level, alpha=.75, cmap='plasma')
-    plt.colorbar()
-    plt.clabel(C, inline=True, fontsize=10, colors='black')
-    plt.xlabel(speed_name)
-    plt.ylabel(torque_name)
+    # 最小二乘法
+    p_lsq = leastsq(residuals_func, p_init, args=(x, y))
+
+    #     print('Fitting Parameters:', p_lsq[0])
+    #     print(residuals_func)
+    #     fitted curve function:
+    def func(x, M):
+        result = 0
+        for i in range(M + 1):
+            result += p_lsq[0][i] * np.power(x, M - i)
+        return result
+
+    #     return func
+    err = (func(x, M) - y).std()
+    print(err)
+
+    labelx = 'Fitted curve {p=' + str(M) + ' | error:' + str(round(err, 3)) + '}'
+
+    # make output formula
+    output = 'Fitted curve: y='
+    for temp in range(M + 1):
+        #         print(temp)
+        if p_lsq[0][temp] > 0:
+            output = output + '+' + str(round(p_lsq[0][temp], 3)) + 'x^[' + str(M - temp) + ']'
+        else:
+            output += str(round(p_lsq[0][temp], 3)) + 'x^[' + str(M - temp) + ']'
+    print(output)
+
+    # 可视化
+#     plt.plot(x_points, real_func(x_points), c='green', lw=2, label='Ideal curve{eg:y=sin(2Π*x)}')
+    plt.plot(x_points, fit_func(p_lsq[0], x_points), c='red', lw=2, label=labelx)
+    plt.plot(x, y, 'o', c='black', label='Real points with noise')
+    plt.legend(loc='upper right')
     plt.title(title)
-
-    # (duty cycle)
     plt.grid(which='both', color='grey', linestyle='--', alpha=0.6)
-    plt.scatter(xpos, ypos, s=hist * bubblesize, alpha=0.3, color='white', linewidth=1, edgecolor='black')
+    # return p_lsq
     plt.show()
+#     plt.savefig("pics\polynomialreg_demo.png")
 
 
 
+# import pandas as pd
+# df=pd.read_csv('C:/Users/lx379/Python_Tony/tony_lab/showcase_data/pol_reg.csv')
+#
+#
+# polynominalreg(df,'para-1','para-2',9,'demo')
 
-df=pd.read_csv('C:/Users/lx379/Python_Tony/PDS_dm_lab/showcase_data/bsfctest.csv')
-bsfcmap_dutycycle(df, 'EngSpd', 'EngTrq', 'BSFC', 150, 150, 4000, 'XX', [180,200,220])
 
 
 
 
 this_is_nothing = 'Below modules are not validated.---------------------👇'
-
-
